@@ -14,6 +14,7 @@ from logging.handlers import TimedRotatingFileHandler
 from typing import Dict
 
 from .task import HeartbeatInfo, HeartbeatTask
+from .auth import StaticCredentialsProvider
 
 try:
     import ssl
@@ -46,7 +47,7 @@ from .timer import NacosTimer, NacosTimerManager
 
 logger = logging.getLogger(__name__)
 
-VERSION = "1.0.0"
+VERSION = "0.1.16"
 
 DEFAULT_GROUP_NAME = "DEFAULT_GROUP"
 DEFAULT_NAMESPACE = ""
@@ -276,7 +277,8 @@ class NacosClient:
             logger.propagate = False
 
     def __init__(self, server_addresses=None, endpoint=None, namespace=None, ak=None,
-                 sk=None, username=None, password=None, logDir=None, log_level=None, log_rotation_backup_count=None):
+                 sk=None, username=None, password=None, logDir=None, log_level=None,
+                 log_rotation_backup_count=None, credentials_provider=None):
         self.server_list = list()
         self.initLog(logDir, log_level, log_rotation_backup_count)
         try:
@@ -308,8 +310,7 @@ class NacosClient:
 
         self.endpoint = endpoint
         self.namespace = namespace or DEFAULT_NAMESPACE or ""
-        self.ak = ak
-        self.sk = sk
+        self.credentials_provider = credentials_provider if credentials_provider else StaticCredentialsProvider(ak, sk)
         self.username = username
         self.password = password
 
@@ -330,7 +331,8 @@ class NacosClient:
         self.process_mgr = None
 
         self.default_timeout = DEFAULTS["TIMEOUT"]
-        self.auth_enabled = self.ak and self.sk
+        credentials = self.credentials_provider.get_credentials()
+        self.auth_enabled = credentials.get_access_key_id() and credentials.get_access_key_secret()
         self.cai_enabled = True
         self.pulling_timeout = DEFAULTS["PULLING_TIMEOUT"]
         self.pulling_config_size = DEFAULTS["PULLING_CONFIG_SIZE"]
@@ -341,7 +343,7 @@ class NacosClient:
         self.proxies = None
         self.logDir = logDir
 
-        self.heartbeats: Dict[str, HeartbeatTask] = {}
+        self.heartbeats = {}
         if self.username and self.password:
             self.get_access_token()
         logger.info("[client-init] endpoint:%s, tenant:%s" % (endpoint, namespace))
@@ -747,6 +749,7 @@ class NacosClient:
             cache_pool[cache_key] = CacheData(cache_key, self)
 
         while cache_list:
+            time.sleep(1)
             unused_keys = set(cache_pool.keys())
             contains_init_key = False
             probe_update_string = ""
@@ -784,10 +787,8 @@ class NacosClient:
                 logger.info("[do-pulling] following keys are changed from server %s" % truncate(str(changed_keys)))
             except NacosException as e:
                 logger.error("[do-pulling] nacos exception: %s, waiting for recovery" % str(e))
-                time.sleep(1)
             except Exception as e:
                 logger.exception("[do-pulling] exception %s occur, return empty list, waiting for recovery" % str(e))
-                time.sleep(1)
 
             for cache_key, cache_data in cache_pool.items():
                 cache_data.is_init = False
@@ -813,35 +814,38 @@ class NacosClient:
         logger.info("[init-pulling] init completed")
 
     def _process_polling_result(self):
-        while True:
-            cache_key, content, md5 = self.notify_queue.get()
-            logger.info("[process-polling-result] receive an event:%s" % cache_key)
-            wl = self.watcher_mapping.get(cache_key)
-            if not wl:
-                logger.warning("[process-polling-result] no watcher on %s, ignored" % cache_key)
-                continue
+        try:
+            while True:
+                cache_key, content, md5 = self.notify_queue.get()
+                logger.info("[process-polling-result] receive an event:%s" % cache_key)
+                wl = self.watcher_mapping.get(cache_key)
+                if not wl:
+                    logger.warning("[process-polling-result] no watcher on %s, ignored" % cache_key)
+                    continue
 
-            data_id, group, namespace = parse_key(cache_key)
-            plain_content = content
+                data_id, group, namespace = parse_key(cache_key)
+                plain_content = content
 
-            params = {
-                "data_id": data_id,
-                "group": group,
-                "namespace": namespace,
-                "raw_content": content,
-                "content": plain_content,
-            }
-            for watcher in wl:
-                if not watcher.last_md5 == md5:
-                    logger.info(
-                        "[process-polling-result] md5 changed since last call, calling %s with changed md5: %s ,params: %s"
-                        % (watcher.callback.__name__, md5, params))
-                    try:
-                        self.callback_tread_pool.apply(watcher.callback, (params,))
-                    except Exception as e:
-                        logger.exception("[process-polling-result] exception %s occur while calling %s " % (
-                            str(e), watcher.callback.__name__))
-                    watcher.last_md5 = md5
+                params = {
+                    "data_id": data_id,
+                    "group": group,
+                    "namespace": namespace,
+                    "raw_content": content,
+                    "content": plain_content,
+                }
+                for watcher in wl:
+                    if not watcher.last_md5 == md5:
+                        logger.info(
+                            "[process-polling-result] md5 changed since last call, calling %s with changed md5: %s ,params: %s"
+                            % (watcher.callback.__name__, md5, params))
+                        try:
+                            self.callback_tread_pool.apply(watcher.callback, (params,))
+                        except Exception as e:
+                            logger.exception("[process-polling-result] exception %s occur while calling %s " % (
+                                str(e), watcher.callback.__name__))
+                        watcher.last_md5 = md5
+        except (EOFError, OSError) as e:
+            logger.exception(f"[process-polling-result] break when receive exception of {type(e)}")
 
     @staticmethod
     def _inject_version_info(headers):
@@ -884,7 +888,8 @@ class NacosClient:
         if not params and not data:
             return
         ts = str(int(time.time() * 1000))
-        ak, sk = self.ak, self.sk
+        # now we have a fixed credentials (access key or sts token)
+        credentials = self.credentials_provider.get_credentials()
 
         sign_str = ""
 
@@ -892,7 +897,7 @@ class NacosClient:
         # config signature
         if "config" == module:
             headers.update({
-                "Spas-AccessKey": ak,
+                "Spas-AccessKey": credentials.get_access_key_id(),
                 "timeStamp": ts,
             })
 
@@ -905,7 +910,9 @@ class NacosClient:
                 sign_str = sign_str + group + "+"
             if sign_str:
                 sign_str += ts
-                headers["Spas-Signature"] = self.__do_sign(sign_str, sk)
+                headers["Spas-Signature"] = self.__do_sign(sign_str, credentials.get_access_key_secret())
+            if credentials.get_security_token():
+                headers["Spas-SecurityToken"] = credentials.get_security_token()
 
         # naming signature
         else:
@@ -922,10 +929,12 @@ class NacosClient:
                 sign_str = ts
 
             params.update({
-                "ak": ak,
+                "ak": credentials.get_access_key_id(),
                 "data": sign_str,
-                "signature": self.__do_sign(sign_str, sk),
+                "signature": self.__do_sign(sign_str, credentials.get_access_key_secret()),
             })
+            if credentials.get_security_token():
+                params.update({"Spas-SecurityToken": credentials.get_security_token()})
 
     def __do_sign(self, sign_str, sk):
         return base64.encodebytes(
@@ -1150,8 +1159,8 @@ class NacosClient:
 
     def send_heartbeat(self, service_name, ip, port, cluster_name=None, weight=1.0, metadata=None, ephemeral=True,
                        group_name=DEFAULT_GROUP_NAME):
-        logger.info("[send-heartbeat] ip:%s, port:%s, service_name:%s, namespace:%s" % (ip, port, service_name,
-                                                                                        self.namespace))
+        logger.debug("[send-heartbeat] ip:%s, port:%s, service_name:%s, namespace:%s" % (ip, port, service_name,
+                                                                                         self.namespace))
         if "@@" not in service_name and group_name:
             service_name = group_name + "@@" + service_name
 
@@ -1186,7 +1195,7 @@ class NacosClient:
             resp = self._do_sync_req("/nacos/v1/ns/instance/beat", None, params, None, self.default_timeout, "PUT",
                                      "naming")
             c = resp.read()
-            logger.info("[send-heartbeat] ip:%s, port:%s, service_name:%s, namespace:%s, server response:%s" %
+            logger.debug("[send-heartbeat] ip:%s, port:%s, service_name:%s, namespace:%s, server response:%s" %
                         (ip, port, service_name, self.namespace, c))
             return json.loads(c.decode("UTF-8"))
         except HTTPError as e:
